#
# getter.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/strategy'
require 'nkf'


module EST

  class SeenGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'boolean'

    def get( mail )
      mail.port.unread
    end
  
  end


  class MultipartGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'boolean'

    def get( mail )
      mail.multipart?
    end
  
  end


  class DateGetter < Strategy

    RC_DEFAULT = { 'format' => '%m/%d %H:%M' }

    VALUE_TYPE = 'string'

    def get( mail )
      if t = mail.date(nil) then
        t.strftime( @rc['format'] )
      else
        ''
      end
    end

  end


  class FromGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'string'

    def get( mail )
      mail.from_phrase(nil) || mail.from('')
    end

  end


  class SubjectGetter < Strategy

    RC_DEFAULT = {
      'sub-regexp' => /\A\s*\[[^:\]]+:(\d+)\]/
    }

    VALUE_TYPE = 'string'

    def get( mail )
      if str = mail.subject(nil) then
        if @rc['sub-regexp'] === str then $1 + $'
                                     else str
                                     end
      else
        ''
      end
    end

  end


  class BodyGetter < Strategy

    RC_DEFAULT = {
      'viewable' => %w( date from subject x-mailer ),
      'hidden'   => %w( received posted message-id  )
    }

    VALUE_TYPE = 'string'

    def hidden_headers( mail )
      rc = @rc['viewable']
      arr = []
      mail.each_header do |k,v|
        arr.push v.inspect unless rc.include? k
      end

      arr
    end


    def viewable_headers( mail )
      ret = @rc['viewable'].collect{|k| mail[k].decoded if mail[k] }
      ret.compact.join("\n") << "\n\n" << mail_to_text( mail ) 
      ret
    end


    def mail_to_text( mail )
      ret = nil
      mtype = mail.main_type.downcase

      case mtype
      when 'text', ''
        ret = mail.body
        case mail.encoding('').downcase
        when '7bit', '8bit', ''
          ;
        when 'base64'
          ret = ret.unpack('m')[0]
        else
          return unknown_encoding( mail )
        end

      when 'multipart'
        mail.parts.each do |m|
          tmp = m.main_type.downcase
          if tmp == 'text' or tmp == '' then
            return mail_to_viewtext(m)
          end
        end
        return unknown_type( mail )

      else
        return unknown_type( mail )
      end

      if NKF.guess( ret ) != NKF::EUC then
        ret = NKF.nkf( '-e', ret )
      end

      # search for "\r\n" from first 200 char
      from = ret.size - 200
      from = from > 0 ? from : 0
      if ret.index( "\r", from ) then
        ret = ret.gsub( "\r\n", "\n" )
      end

      ret
    end


    def unknown_encoding( mail )
      "cannot decode this mail\n" +
      mail['content-transfer-encoding'].decoded
    end

    def unknown_type( mail )
      "unknown content type\n" +
      mail['content-type'].decoded
    end

  end

end
