#
# rr.y  ---  confp.rb racc file
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

class ConfigFileParser

rule

toplevel    :
                {
                    @hstack = [ {} ]
                }
            h_lines
                {
                    result = @hstack[0]
                }
            ;

h_lines     : h_line
            | h_lines EOL h_line
            ;

h_line      : # none
            | '=' STRING
                {
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[1], 'config' ) )
                    set_value0 hash
                }
            | STRING sep strlist
                {
                    set_value1 result, val[2]
                }
            | string block
                {
                    set_value1 result, val[1]
                }
            | STRING QUOTED block
                {
                    set_value2 result, val[1], val[2]
                }
            | STRING QUOTED '=' STRING
                {
                    hash = ConfigFileParser.parse(
                      CORE.search_file( val[3], 'config' ) )
                    set_value2 result, val[1], hash
                }
            ;

sep         : ':'
            | '='
            ;

block       : hash_block | array_block ;

hash_block  : '{'
                {
                    @hstack.push Hash.new
                }
            h_lines '}'
                {
                    result = @hstack.pop
                }
            ;

array_block : '[' a_lines ']'
                {
                    result = val[1]
                }
            ;

a_lines     : # none
                {
                    result = []
                }
            | a_lines EOL a_line
                {
                    result.push val[2] if val[2]
                }
            ;

a_line      : # none
            | strlist
            | block
            ;

strlist     : string
            | strlist string
                {
                    result = [result] unless Array === result
                    result.push val[1]
                }
            ;

string      : STRING | QUOTED ;

end

---- header
#
# confp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/confs'

module EST

---- inner

  class << self
    def parse( fn )
      new.parse( fn )
    end
  end

  def initialize( dflag = false )
    @yydebug = dflag
  end

  def parse( fn )
    unless File.exist? fn then
      raise ArgumentError, "config file #{fn} not exist"
    end
    str = nil
    File.open( fn ) {|f| str = f.read }
    @scanner = ConfigFileScanner.new( str )

    do_parse
  end

  private

  def next_token
    @scanner.scan
  end

  def on_error( et, ev, _values )
    raise ParseError, <<S

parse error in line #{@scanner.lineno}
S
  end


  def set_value0( hash )
    h = @hstack[-1]
    h.update hash
  end
    
  def set_value1( key, val )
    h = @hstack[-1]
    h[ key ] = val
  end

  def set_value2( key1, key2, val )
    h = @hstack[-1]
    if tmp = h[ key1 ] then
      tmp[ key2 ] = val
    else
      h[ key1 ] = { key2 => val }
    end
  end

---- footer

end   # module EST
